import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;

import com.sun.java.swing.*;
import com.sun.java.swing.preview.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import sv.ui.*;
import sv.drawer.*;
import sv.kernel.CommandSet;

/** 
 * DrawerUI.java
 * Drawing tool interface
 * @version 1.0     11/10/97
 * @author Byeongseob Ki
 */


public class DrawerUI extends Frame implements ActionListener, ItemListener {
  svserver svServer;

  public DrawerUI(svserver svServer) {
    super("Drawing tool"); 
    this.svServer = svServer;

    makeGUI();
  }

  ButtonGroup groups, pointergroup;
  JComboBox namecb, sizecb, stylecb;  
  JToggleButton groupdummybtn, pointerdummybtn, linebtn, pointerbtn, rectbtn;
  SelectedCanvas canvas;

  void makeGUI() {
     setLayout(new BorderLayout());

     Panel centerPanel = new Panel();
     centerPanel.setLayout(new BorderLayout());
    
     JPanel btnPanel = new JPanel();
     btnPanel.setLayout(new GridLayout(2, 6, 2, 2));
     btnPanel.setBorder(SVUIC.systemBorder);

     groups = new ButtonGroup();

     JToggleButton button;
     JButton jbtn;
     ImageIcon icon;

     icon = svServer.loadImageIcon(getClass().getResource("icons/line.gif"), "line");
     linebtn = new JToggleButton(icon);
     groups.add(linebtn);
     linebtn.setToolTipText("Draw line object");
     linebtn.setActionCommand("line");
     linebtn.setMargin(new Insets(0, 0, 0, 0));
     linebtn.addActionListener(this);
     btnPanel.add(linebtn);
	
     icon = svServer.loadImageIcon(getClass().getResource("icons/bezier.gif"), "bezier");
     button = new JToggleButton(icon);
     groups.add(button);
     button.setToolTipText("Draw bezier curve");
     button.setActionCommand("bezier");
     button.setMargin(new Insets(0, 0, 0, 0));
     button.addActionListener(this);
     btnPanel.add(button);

     icon = svServer.loadImageIcon(getClass().getResource("icons/mouse.gif"), "mouse");
     button = new JToggleButton(icon);
     groups.add(button);
     button.setToolTipText("Draw line followed by your mouse position");
     button.setActionCommand("mouse");
      button.setMargin(new Insets(0, 0, 0, 0));
     button.addActionListener(this);
     btnPanel.add(button);

     icon = svServer.loadImageIcon(getClass().getResource("icons/oval.gif"), "oval");
     button = new JToggleButton(icon);
     groups.add(button);
     button.setToolTipText("Draw oval object");
     button.setActionCommand("oval");
      button.setMargin(new Insets(0, 0, 0, 0));
     button.addActionListener(this);
     btnPanel.add(button);

     icon = svServer.loadImageIcon(getClass().getResource("icons/rect.gif"), "rect");
     rectbtn = new JToggleButton(icon);
     groups.add(rectbtn);
     rectbtn.setToolTipText("Draw rectangle object");
     rectbtn.setActionCommand("rect");
     rectbtn.setMargin(new Insets(0, 0, 0, 0));
     rectbtn.addActionListener(this);
     btnPanel.add(rectbtn);

     icon = svServer.loadImageIcon(getClass().getResource("icons/text.gif"), "text");
     button = new JToggleButton(icon);
     groups.add(button);
     button.setToolTipText("Draw text object");
     button.setActionCommand("text");
      button.setMargin(new Insets(0, 0, 0, 0));
     button.addActionListener(this);
     btnPanel.add(button);

     icon = svServer.loadImageIcon(getClass().getResource("icons/more.gif"), "more");
     button = new JToggleButton(icon);
     groups.add(button);
     button.setToolTipText("Load image");
     button.setActionCommand("image");
     button.setMargin(new Insets(0, 0, 0, 0));
     button.addActionListener(this);
     btnPanel.add(button);


     groupdummybtn = new JToggleButton();
     groupdummybtn.setActionCommand("groups dummy");
     groupdummybtn.addActionListener(this);
     groups.add(groupdummybtn);


     icon = svServer.loadImageIcon(getClass().getResource("icons/color1.gif"), "color");
     jbtn = new JButton(icon);
     jbtn.setToolTipText("Invoke line color selector");
     jbtn.setActionCommand("line color");
     jbtn.addActionListener(this);
     jbtn.setMargin(new Insets(0, 0, 0, 0));
     btnPanel.add(jbtn);

   
     icon = svServer.loadImageIcon(getClass().getResource("icons/color2.gif"), "color");
     jbtn = new JButton(icon);
     jbtn.setToolTipText("Invoke fill color selector");
     jbtn.setActionCommand("fill color");
     jbtn.setMargin(new Insets(0, 0, 0, 0));
     jbtn.addActionListener(this);
     btnPanel.add(jbtn);

     icon = svServer.loadImageIcon(getClass().getResource("icons/color3.gif"), "color");
     jbtn = new JButton(icon);
     jbtn.setToolTipText("Invoke text color selector");
     jbtn.setActionCommand("text color");
     jbtn.setMargin(new Insets(0, 0, 0, 0));
     jbtn.addActionListener(this);
     btnPanel.add(jbtn);


     icon = svServer.loadImageIcon(getClass().getResource("icons/pen.gif"), "pen");
     jbtn = new JButton(icon);
     jbtn.setToolTipText("Invoke pen and line selector");
     jbtn.setActionCommand("pen");
     jbtn.addActionListener(this);
     jbtn.setMargin(new Insets(0, 0, 0, 0));
     btnPanel.add(jbtn);

     pointergroup = new ButtonGroup();
     icon = svServer.loadImageIcon(getClass().getResource("icons/pointer.gif"), "pointer");
     pointerbtn = new JToggleButton(icon);
     pointergroup.add(pointerbtn);
     pointerbtn.setToolTipText("To make polygon");
     pointerbtn.setActionCommand("pointer");
     pointerbtn.addActionListener(this);
     pointerbtn.setMargin(new Insets(0, 0, 0, 0));
     btnPanel.add(pointerbtn);

     pointerdummybtn = new JToggleButton();
     pointerdummybtn.setActionCommand("pointer dummy");
     pointerdummybtn.addActionListener(this);
     pointergroup.add(pointerdummybtn);


     centerPanel.add("Center", btnPanel);

     JPanel pp = new JPanel();
     pp.setLayout(new BorderLayout());
     pp.setBorder(new TitledBorder("Selected pen & colors"));
     pp.add("Center", canvas = new SelectedCanvas(this));
     canvas.setSize(pp.getSize().width-20, 30);

     centerPanel.add("South", pp);
	
     ///////////////////////////////////////////////////////////////
     Panel lowerPanel = new Panel();
     lowerPanel.setLayout(new GridLayout(2, 1));
  
     Panel fontPanel = new Panel();
     Panel finalPanel = new Panel();

     fontPanel.setLayout(new BoxLayout(fontPanel, BoxLayout.X_AXIS));
 
     namecb = new JComboBox();
     String[] fonts = getToolkit().getFontList();

     for(int i=0; i<fonts.length; i++)
         namecb.addItem(fonts[i]);

     for (int i=0; i<fonts.length; i++) {
       if (fonts[i].equals("Helvetica")) {
          namecb.setSelectedItem("Helvetica");
          break;
       }
       if (fonts[i].equals("Dialog")) {
          namecb.setSelectedItem("Dialog");
          break;
       }
     }

     sizecb = new JComboBox();
     sizecb.addItem(" 8");
     sizecb.addItem("10");
     sizecb.addItem("12");
     sizecb.addItem("14");
     sizecb.addItem("18");
     sizecb.addItem("24");
     sizecb.addItem("36");
     sizecb.setSelectedItem("10");

     stylecb = new JComboBox();
     stylecb.addItem("Plain");
     stylecb.addItem("Bold");
     stylecb.addItem("Italic");

     namecb.addItemListener(this);
     sizecb.addItemListener(this);
     stylecb.addItemListener(this);

     fontPanel.add(namecb);
     fontPanel.add(sizecb);
     fontPanel.add(stylecb);

     finalPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
     Button b = new Button("Quit");
     b.addActionListener(this);
     finalPanel.add(b);

     ///////////////////////////////////////////////////////////////
     lowerPanel.add(fontPanel);
     lowerPanel.add(finalPanel);

     add("Center", centerPanel);
     add("South", lowerPanel);

  }

  public void actionPerformed(ActionEvent e) {
     String command = e.getActionCommand();
     
     if (e.getSource() instanceof JToggleButton) {
       if (command.equals("line")) {
	 if (pointerbtn.isSelected()) 
           svServer.shapes.setShapeType(ShapeProperties.MULTILINE);
         else
           svServer.shapes.setShapeType(ShapeProperties.LINE);

       } else if (command.equals("bezier")) {
         if (pointerbtn.isSelected()) 
           clearPointerToggle();
         svServer.shapes.setShapeType(ShapeProperties.BEZIER);

       } else if (command.equals("mouse")) {
         if (pointerbtn.isSelected()) 
           clearPointerToggle();
         svServer.shapes.setShapeType(ShapeProperties.MOUSE);

       } else if (command.equals("oval")) {
         if (pointerbtn.isSelected()) 
           clearPointerToggle();
         svServer.shapes.setShapeType(ShapeProperties.OVAL);

       } else if (command.equals("rect")) {
	 if (pointerbtn.isSelected()) 
           svServer.shapes.setShapeType(ShapeProperties.POLYGON);
         else
           svServer.shapes.setShapeType(ShapeProperties.RECTANGLE);

       } else if (command.equals("text")) {
         if (pointerbtn.isSelected()) 
           clearPointerToggle();
         svServer.shapes.setShapeType(ShapeProperties.TEXT);

       } else if (command.equals("image")) {
         if (pointerbtn.isSelected()) 
           clearPointerToggle();
         svServer.shapes.setShapeType(ShapeProperties.IMAGE);

       } else if (command.equals("groups dummy")) {
         svServer.shapes.setShapeType(ShapeProperties.NONE);
         return;                                      // ok finish it

       } else if (command.equals("pointer")) {
         if (linebtn.isSelected()) 
            svServer.shapes.setShapeType(ShapeProperties.MULTILINE);
         else if (rectbtn.isSelected()) 
            svServer.shapes.setShapeType(ShapeProperties.POLYGON);
         else 
            clearGroupToggle();
       } 

     } else if (e.getSource() instanceof JButton) {

       if (command.equals("line color")) {
          Color c = 
              JColorChooser.showDialog(this, 
                          "Line Color",  svServer.shapes.getLineColor());
           if (c == null)
               return;
           String args[] = { String.valueOf(c.getRGB()) };
           svServer.fmanager.actionGlobal(CommandSet.WLINECOLOR, args, true);
           canvas.repaint();
       } else if (command.equals("fill color")) {
          Color c = 
              JColorChooser.showDialog(this, 
                          "Fill Color",  svServer.shapes.getFillColor());
           if (c == null)
               return;
           String args[] = { String.valueOf(c.getRGB()) };
           svServer.fmanager.actionGlobal(CommandSet.WFILLCOLOR, args, true);
           canvas.repaint();

       } else if (command.equals("text color")) {
          Color c = 
              JColorChooser.showDialog(this, 
                          "Text Color",  svServer.shapes.getTextColor());
           if (c == null)
               return;
           String args[] = { String.valueOf(c.getRGB()) };
           svServer.fmanager.actionGlobal(CommandSet.WTEXTCOLOR, args, true);
           canvas.repaint();

       } else if (command.equals("pen")) {
          LineChooser chooser = 
                new LineChooser(this, 
                       svServer.shapes.getLineType(),
                       svServer.shapes.getLineWidth(),
                       svServer.shapes.getDashSize(),
                       svServer.shapes.getLineColor());

          chooser.show();

          int line_type = chooser.getLineType();
          if (line_type < 0) return;

          String[] args = new String[3];
          args[0] = String.valueOf(line_type);
          args[1] = String.valueOf(chooser.getLineWidth());
          args[2] = String.valueOf(chooser.getDashSize());
          svServer.fmanager.actionGlobal(CommandSet.WPEN, args, true);
          canvas.repaint();
       } 

     } else if (command.equals("Quit")) { 
        svServer.mainb.drawBtn.setEnabled(true);
        svServer.fmanager.setDrawingMode(false);
        this.dispose();
        svServer.dtool = null;
     } 
  }

  public void clearAllToggleButtons() {
     groupdummybtn.doClick();
     pointerdummybtn.doClick();
  }

  private void clearPointerToggle() {
     pointerdummybtn.doClick();
  }

  private void clearGroupToggle() {
     groupdummybtn.doClick();
  }


  /**
   * overrided method (for list)
   */
  public void itemStateChanged(ItemEvent e) {
     if (e.getSource() instanceof JComboBox &&
                        e.getStateChange() == ItemEvent.SELECTED) {
        String name;
        int style, size;

        name = ((String)namecb.getSelectedItem()).trim();
        int stylep = stylecb.getSelectedIndex();
        int sizep = sizecb.getSelectedIndex();

        int[] sizearr = {8, 10, 12, 14, 18, 24, 36};
        int[] stylearr = {Font.PLAIN, Font.BOLD, Font.ITALIC };

        size = sizearr[sizep];
        style = stylearr[stylep];

        String[] args = new String[3];
        args[0] = name;
        args[1] = String.valueOf(style);
        args[2] = String.valueOf(size);
        svServer.fmanager.actionGlobal(CommandSet.WTEXTFONT, args, true);

	//System.out.println(name + " " + style + " " + size); 
     }
  }


  class SelectedCanvas extends Canvas {
    svserver parent;

    public SelectedCanvas(DrawerUI p) {
       parent = p.svServer;
    }
   
    public void paint(Graphics g) {
       Dimension d = getSize();

       g.setColor(Color.white);
       g.fillRect(0, 0, d.width, d.height);

       int cy = d.height / 2;

       int sx = 15;
       int ex = 95;

       int[] xpts = new int[3];
       int[] ypts = new int[3];
       int dummy1, dummy2;

       int lwid = parent.shapes.getLineWidth();
       Color lcol = parent.shapes.getLineColor();
       int dashsz = parent.shapes.getDashSize();

       switch (parent.shapes.getLineType()) {
        case LineShape.NOARROW_LINE:
          SVUIC.drawLine(g, sx, cy, ex, cy, lwid, lcol);
          break;

        case LineShape.RIGHTFILLARROW_LINE:
          dummy1 = ex-lwid-10;

          SVUIC.drawLine(g, sx, cy, dummy1, cy, lwid, lcol);
          xpts[0] = dummy1;
          xpts[1] = ex;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.fillPolygon(xpts, ypts, 3);

          break;

        case LineShape.RIGHTARROW_LINE:
          dummy1 = ex-lwid-10;

          SVUIC.drawLine(g, sx, cy, dummy1, cy, lwid, lcol);
          xpts[0] = dummy1;
          xpts[1] = ex;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.drawPolygon(xpts, ypts, 3);

          break;

        case LineShape.LEFTFILLARROW_LINE:
          dummy1 = sx+lwid+10;

          SVUIC.drawLine(g, dummy1, cy, ex, cy, lwid, lcol);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.fillPolygon(xpts, ypts, 3);

          break;

        case LineShape.LEFTARROW_LINE:
          dummy1 = sx+lwid+10;

          SVUIC.drawLine(g, dummy1, cy, ex, cy, lwid, lcol);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.drawPolygon(xpts, ypts, 3);

          break;

        case LineShape.LRFILLARROW_LINE:
          dummy1 = sx+lwid+10;
          dummy2 = ex-lwid-10;

          SVUIC.drawLine(g, dummy1, cy, dummy2, cy, lwid, lcol);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.fillPolygon(xpts, ypts, 3);

          xpts[0] = dummy2;
          xpts[1] = ex;
          xpts[2] = dummy2;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.fillPolygon(xpts, ypts, 3);


          break;

        case LineShape.LRARROW_LINE:
          dummy1 = sx+lwid+10;
          dummy2 = ex-lwid-10;

          SVUIC.drawLine(g, dummy1, cy, dummy2, cy, lwid, lcol);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.drawPolygon(xpts, ypts, 3);

          xpts[0] = dummy2;
          xpts[1] = ex;
          xpts[2] = dummy2;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.drawPolygon(xpts, ypts, 3);

          break;


        case LineShape.NOARROW_DLINE:
          SVUIC.drawDottedLine(g, sx, cy, ex, cy, lwid, dashsz, lcol);
          break;

        case LineShape.RIGHTFILLARROW_DLINE:
          dummy1 = ex-lwid-10;

          SVUIC.drawDottedLine(g, sx, cy, dummy1, cy, lwid, dashsz, lcol);
          xpts[0] = dummy1;
          xpts[1] = ex;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.fillPolygon(xpts, ypts, 3);

          break;

        case LineShape.RIGHTARROW_DLINE:
          dummy1 = ex-lwid-10;

          SVUIC.drawDottedLine(g, sx, cy, dummy1, cy, lwid, dashsz, lcol);
          xpts[0] = dummy1;
          xpts[1] = ex;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.drawPolygon(xpts, ypts, 3);

          break;

        case LineShape.LEFTFILLARROW_DLINE:
          dummy1 = sx+lwid+10;

          SVUIC.drawDottedLine(g, dummy1, cy, ex, cy, lwid, dashsz, lcol);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.fillPolygon(xpts, ypts, 3);

          break;

        case LineShape.LEFTARROW_DLINE:
          dummy1 = sx+lwid+10;

          SVUIC.drawDottedLine(g, dummy1, cy, ex, cy, lwid, dashsz, lcol);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.drawPolygon(xpts, ypts, 3);

          break;

        case LineShape.LRFILLARROW_DLINE:
          dummy1 = sx+lwid+10;
          dummy2 = ex-lwid-10;

          SVUIC.drawDottedLine(g, dummy1, cy, dummy2, cy, lwid, dashsz, lcol);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.fillPolygon(xpts, ypts, 3);

          xpts[0] = dummy2;
          xpts[1] = ex;
          xpts[2] = dummy2;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.fillPolygon(xpts, ypts, 3);


          break;

        case LineShape.LRARROW_DLINE:
          dummy1 = sx+lwid+10;
          dummy2 = ex-lwid-10;

          SVUIC.drawDottedLine(g, dummy1, cy, dummy2, cy, lwid, dashsz, lcol);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.drawPolygon(xpts, ypts, 3);

          xpts[0] = dummy2;
          xpts[1] = ex;
          xpts[2] = dummy2;
          ypts[0] = cy - lwid - 5;
          ypts[1] = cy;
          ypts[2] = cy + lwid + 5;
          g.setColor(lcol);
          g.drawPolygon(xpts, ypts, 3);

          break;

       }

       int w = (d.width - 135) / 2;
       int h = d.height - 10;

       g.setColor(parent.shapes.getFillColor());
       g.fillRect(120, 5, w, h);
   
       g.setColor(parent.shapes.getTextColor());
       g.fillRect(120+w, 5, w, h);
       
    }

    public void update(Graphics g) {
       paint(g);
    }

  }
} // end of class




